@@ -16,7 +16,15 @@ from bokeh.models.tools import PanTool, BoxZoomTool, WheelZoomTool, ZoomInTool,
 from bokeh.palettes import Viridis256
 from bokeh.io import output_notebook
 from .util import Audio
-output_notebook()
+
+
+try:
+    ipyname = get_ipython().__class__.__name__
+    ipymodule = get_ipython().__class__.__module__
+    if ipyname == 'ZMQInteractiveShell' or ipymodule == 'google.colab._shell':
+        output_notebook()
+except NameError:
+    pass
 
 
 def get_samples_and_rate(input_signal, samplerate):
@@ -50,13 +58,18 @@ def _get_compatible_samples(ref_signal, other):
         other_samples = other.samples
     else:
         other_samples = np.asarray(other)
-    if other_samples.size != ref_signal.samples.size:
-        raise ValueError('Signals need to have the same size')
     if np.issubdtype(other_samples.dtype, np.number):
         return other_samples
     raise TypeError('Only TimeSignals or numeric iterables are supported as operands, not {}'.format(type(other)))
 
 
+def _get_compatible_size_samples(ref_signal, other):
+    other_samples = _get_compatible_samples(ref_signal, other)
+    if other_samples.size != ref_signal.samples.size:
+        raise ValueError('Signals need to have the same size')
+    return other_samples
+
+
 def get_both_samples_and_rate(input_signal1, input_signal2, samplerate=None):
     samples1, samplerate1 = get_samples_and_rate(input_signal1, samplerate)
     samples2, samplerate2 = get_samples_and_rate(input_signal2, samplerate)
@@ -105,7 +118,7 @@ class TimeSignal(Signal):
             self.samplerate = in_data._samplerate
         else:
             if isinstance(in_data, Spectrum):
-                self.samples = fft.irfft(in_data.spectrum, in_data._fft_size, norm='forward')
+                self.samples = fft.irfft(in_data.spectrum, in_data._num_bins, norm='forward')
                 self.samplerate = in_data._samplerate
             else:
                 if samplerate is None:
@@ -134,14 +147,14 @@ class TimeSignal(Signal):
     def __add__(self, other):
         if np.isscalar(other):
             return same_type_as(self.samples + other, self)
-        other_samples = _get_compatible_samples(self, other)
+        other_samples = _get_compatible_size_samples(self, other)
         return same_type_as(self.samples + other_samples, self)
 
 
     def __mul__(self, other):
         if np.isscalar(other):
             return same_type_as(self.samples * other, self)
-        other_samples = _get_compatible_samples(self, other)
+        other_samples = _get_compatible_size_samples(self, other)
         return same_type_as(self.samples * other_samples, self)
 
 
@@ -222,7 +235,6 @@ class AudioSignal(TimeSignal):
             'active_scroll': 'auto',
             'toolbar_location': 'above',
             'tooltips': [('time [s]', '@x{0.000}'), ('amplitude', '@y{0.000}')],
-            'output_backend': 'webgl',
         }
         kwargs = {**default_args, **fig_args}
         if kwargs['tooltips'] is not None:
@@ -234,7 +246,7 @@ class AudioSignal(TimeSignal):
 
 class Spectrum(Signal):
 
-    def __init__(self, in_data, samplerate=None, num_bins=None, exponent=1, dB=True):
+    def __init__(self, in_data, dB=True, num_bins=None, exponent=1, norm_single_side_band=False, samplerate=None):
         samples, samplerate = get_samples_and_rate(in_data, samplerate)
 
         if num_bins is None:
@@ -243,6 +255,7 @@ class Spectrum(Signal):
         self._samplerate = samplerate
         self._num_bins = num_bins
         self.exponent = exponent
+        self._norm_single_side_band = norm_single_side_band
         self.dB = dB
 
         spectrum = fft.rfft(samples, num_bins, norm='forward')
@@ -250,8 +263,13 @@ class Spectrum(Signal):
         self.phase = np.angle(spectrum)
         self.frequencies = np.arange(len(spectrum)) * samplerate / num_bins
 
+        if norm_single_side_band:
+            if self._num_bins % 2 == 0:
+                self.magnitude[1:-1] *= sqrt(2)
+            else:
+                self.magnitude[1:] *= sqrt(2)
         if dB:
-            self.magnitude = 20 * np.log10(np.clip(self.magnitude, 1e-12, None))
+            self.magnitude = 20 * np.log10(np.clip(self.magnitude, 1e-6, None))
         else:
             self.magnitude **= exponent
 
@@ -274,7 +292,6 @@ class Spectrum(Signal):
             'active_scroll': 'auto',
             'toolbar_location': 'above',
             'tooltips': [('frequency [Hz]', '@x{0.0}'), ['magnitude', '@y{0.000}']],
-            'output_backend': 'webgl',
         }
         if self.exponent == 2 or self.dB:
             default_args['y_axis_label'] = 'power'
@@ -307,7 +324,9 @@ class Spectrum(Signal):
             power_per_freq = 10 ** (self.magnitude / 10)
         else:
             power_per_freq = self.magnitude ** (2/self.exponent)
-        if self._fft_size % 2 == 0:
+        if self._norm_single_side_band:
+            power = np.sum(power_per_freq)
+        elif self._num_bins % 2 == 0:
             power = power_per_freq[0] + 2*np.sum(power_per_freq[1:-1]) + power_per_freq[-1]
         else:
             power = power_per_freq[0] + 2*np.sum(power_per_freq[1:])
@@ -340,12 +359,20 @@ class Spectrum(Signal):
             magnitude = 10 ** (self.magnitude / 20)
         else:
             magnitude = self.magnitude ** (1/self.exponent)
+        if self._norm_single_side_band:
+            if self._num_bins % 2 == 0:
+                self.magnitude[1:-1] /= sqrt(2)
+            else:
+                self.magnitude[1:] /= sqrt(2)
         return magnitude * np.exp(1j*self.phase)
 
 
 class PowerSpectrum(Spectrum):
-    def __init__(self, in_data, samplerate=None, num_bins=None, dB=True):
-        super().__init__(in_data, samplerate=samplerate, num_bins=num_bins, exponent=2, dB=dB)
+    def __init__(self, in_data, **kwargs):
+        exponent = kwargs.pop('exponent', None)
+        if exponent is not None and exponent != 2:
+            warnings.warn('Magnitude exponent is automatically set to 2 for a PowerSpectrum')
+        super().__init__(in_data, exponent=2, **kwargs)
 
 
 class Spectrogram(Signal):
@@ -367,7 +394,7 @@ class Spectrogram(Signal):
         self.phase = np.angle(spectrogram)
 
         if dB:
-            self.magnitude = 20 * np.log10(np.clip(self.magnitude, 1e-12, None))
+            self.magnitude = 20 * np.log10(np.clip(self.magnitude, 1e-6, None))
         else:
             self.magnitude **= exponent
 
@@ -389,7 +416,6 @@ class Spectrogram(Signal):
             'active_scroll': 'auto',
             'toolbar_location': 'above',
             'tooltips': [('time [s]', '@time{0.000}'), ('frequency [Hz]', '@frequency{0.0}'), ['magnitude', '@magnitude{0.000}']],
-            'output_backend': 'webgl',
         }
 
         if self.exponent == 2 or self.dB:
